package main

import (
	"errors"
	"fmt"
	"golang.org/x/sys/windows/registry"
	"io"
	"log"
	"os"
	"os/exec"
	"path/filepath"
	"regexp"
)

/* ----------------------------------- 从注册表遍历可执行文件路径,可用os/exec包的LookPath代替 -----------------------------------*/
func checkpath(name string) (path string, errs error) {
	//func OpenKey(k Key, path string, access uint32) (Key, error)
	//OpenKey打开一个新键，其路径名相对于键K

	//func (k Key) ReadValueNames(n int) ([]string, error)
	//ReadValueNames返回键k的值名称。参数n控制返回的名称的数量，类似于os.File.Readdirnames的工作方式。

	//func (k Key) GetStringValue(name string) (val string, valtype uint32, err error)
	k, err := registry.OpenKey(registry.CURRENT_USER, `Software\Classes\Local Settings\Software\Microsoft\Windows\Shell\MuiCache`, registry.READ)
	if err != nil {
		log.Fatal(err)
	}
	defer k.Close()
	s, err := k.ReadValueNames(0) //s 是遍历值名称切片
	if err != nil {
		log.Fatal(err)
	}
	path = ""
	errs = errors.New(name + " " + "not find")
	for i := 0; i < len(s); i++ {
		data, valtype, err := k.GetStringValue(s[i])
		if valtype != registry.SZ { //先判断值类型，如果值类型不是SZ，跳过
			continue
		} else if err != nil {
			log.Fatal(err)
		} else if data == name {
			path = s[i]
			errs = nil
			break
		}
	}
	return path, errs
}

/* ----------------------------------- 正则表达式提取路径，可用path包的Dir函数替代 -----------------------------------*/
func regcheck(s string) (regs string, errs error) {
	pattern := `(.*)\\.*.exe`
	reg := regexp.MustCompile(pattern)
	pathmu := reg.FindStringSubmatch(s)
	if pathmu == nil {
		errs = errors.New("正则表达式bug")
		return "", errs
	}
	return pathmu[1], errs
}

/* ----------------------------------- 复制文件函数 -----------------------------------*/
func copyFile(dstName, srcName string) (written int64, err error) {
	src, err := os.Open(srcName)
	if err != nil {
		log.Fatal(err)
	}
	defer src.Close()
	dst, err := os.Create(dstName)
	if err != nil {
		log.Fatal(err)
	}
	defer dst.Close()
	return io.Copy(dst, src)
}

/* ----------------------------------- 检测文件夹存不存在，防止旧版本卸载注册表残留 -----------------------------------*/
func Exists(path string) bool {
	_, err := os.Stat(path)
	if err != nil {
		if os.IsExist(err) {
			return true
		}
		return false
	}
	return true
}

/* ----------------------------------- 编辑goland.exe.vmoptions文件和goland64.exe.vmoptions文件 -----------------------------------*/
func editFile(path, name string) {
	file, err := os.OpenFile(path+"\\"+name+".exe.vmoptions", os.O_APPEND|os.O_WRONLY, 0600)
	if err != nil {
		log.Fatal(err)
	}
	defer file.Close()
	if _, err := file.WriteString("\r\n-javaagent:" + path + "\\JetbrainsIdesCrack-4.2-release-enc.jar"); err != nil {
		log.Fatal(err)
	}
}
func main() {
	//file, _ := exec.LookPath(os.Args[0]) //os.Args[0]是程序名,这个是为了获取程序当前运行路径
	//path, _ := filepath.Abs(file)        //Abs函数返回path代表的绝对路径，如果path不是绝对路径，会加入当前工作目录以使之成为绝对路径。因为硬链接的存在，不能保证返回的绝对路径是唯一指向该地址的绝对路径
	//println(path)
	t := true
	fmt.Println("1：GoLand")
	fmt.Println("2：PyChram")
	fmt.Println("3：WebStorm")
	fmt.Println("0：退出程序")
	for t {
		var n int
		fmt.Print("请选择：")
		fmt.Scan(&n)
		if n == 0 {
			fmt.Println("程序退出！！！")
			os.Exit(0)
		} else if n == 1 {
			/* ----------------------------------- GolandD的处理 -----------------------------------*/
			//path, _ := checkpath("GoLand")  //更新后注册表无键值对bug
			file, _ := exec.LookPath("golang.exe")
			path, _ := filepath.Abs(file)
			fmt.Println("GoLand安装路径：", path)
			golandpath, err := regcheck(path)
			if err != nil {
				log.Fatal(err)
			}
			if _, err := copyFile(golandpath+"\\"+"JetbrainsIdesCrack-4.2-release-enc.jar", "JetbrainsIdesCrack-4.2-release-enc.jar"); err != nil {
				log.Fatal(err)
			}
			editFile(golandpath, "golang")
			editFile(golandpath, "golang64")
			fmt.Println("请复制注册码输入")
		} else if n == 2 {
			/* ----------------------------------- Pycharm的处理 -----------------------------------*/
			//path, _ := checkpath("Pycharm")  //更新后注册表无键值对bug
			file, _ := exec.LookPath("pycharm.exe")
			path, _ := filepath.Abs(file)
			fmt.Println("Pycharm安装路径：", path)
			pycharmpath, err := regcheck(path)
			fmt.Println(pycharmpath)
			if err != nil {
				log.Fatal(err)
			}
			if _, err := copyFile(pycharmpath+"\\"+"JetbrainsIdesCrack-4.2-release-enc.jar", "JetbrainsIdesCrack-4.2-release-enc.jar"); err != nil {
				log.Fatal(err)
			}
			editFile(pycharmpath, "pycharm")
			editFile(pycharmpath, "pycharm64")
			fmt.Println("请复制注册码输入")
		} else if n == 3 {
			/* ----------------------------------- Webstorm的处理 -----------------------------------*/
			//path, _ := checkpath("Webstorm")  //更新后注册表无键值对bug
			file, _ := exec.LookPath("webstorm.exe")
			path, _ := filepath.Abs(file)
			fmt.Println("Webstorm安装路径：", path)
			webstormpath, err := regcheck(path)
			if err != nil {
				log.Fatal(err)
			}
			if _, err := copyFile(webstormpath+"\\"+"JetbrainsIdesCrack-4.2-release-enc.jar", "JetbrainsIdesCrack-4.2-release-enc.jar"); err != nil {
				log.Fatal(err)
			}
			editFile(webstormpath, "webstorm")
			editFile(webstormpath, "webstorm64")
			fmt.Println("请复制注册码输入")
		} else {
			fmt.Println("请输入正确选择！！！")
			t = false
		}
	}
	//path, err := checkpath("GoLand")
	//
	//if err != nil {
	//	log.Fatal(err)
	//}
	//fmt.Println(path)

}
